# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2013 Brian Douglass bhdouglass@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

from gi.repository import Gtk, GObject # pylint: disable=E0611

from indicator_remindor import helpers

from remindor_common import datetimeutil
from remindor_common.helpers import SimpleDialogInfo

class SimpleDialog(Gtk.Dialog):
    __gtype_name__ = 'SimpleDialog'
    __gsignals__ = {
        'reminder_added' : (GObject.SIGNAL_RUN_LAST, GObject.TYPE_NONE,
            (GObject.TYPE_STRING,))
    }

    def __new__(cls):
        builder = helpers.get_builder('SimpleDialog')
        new_object = builder.get_object('simple_dialog')
        new_object.finish_initializing(builder)
        return new_object

    def finish_initializing(self, builder):
        self.builder = builder
        self.ui = builder.get_ui(self)
        self.set_default_size(350, -1)

        self.reminder_e = self.builder.get_object('reminder_entry')
        self.reminder_e.connect('icon-release', self.reminder_clear)
        self.reminder_e.connect('changed', self.validate)

        self.info = SimpleDialogInfo(helpers.database_file())
        self.show_label_error = False

    def validate(self, widget, data=None):
        text = self.reminder_e.get_text()
        (valid, date_s, time_s, label) = self.info.validate(text)

        if label == text and not self.show_label_error:
            valid = True

        if label != text and not self.show_label_error:
            self.show_label_error = True

        if valid:
            self.reminder_e.set_icon_from_stock(0, None)
        else:
            self.reminder_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)

    def reminder_clear(self, widget, pos, event, data=None):
        self.reminder_e.set_text('')

    def on_btn_help_clicked(self, widget, data=None):
        helpers.show_uri(self, helpers.get_help_uri('simple'))

    def on_btn_add_clicked(self, widget, data=None):
        id = self.info.reminder(self.reminder_e.get_text())
        if id != None:
            self.emit('reminder_added', id)
            self.destroy()
        else:
            self.reminder_e.set_icon_from_stock(0, Gtk.STOCK_DIALOG_ERROR)
            self.show_label_error = True

    def on_btn_cancel_clicked(self, widget, data=None):
        self.destroy()
